# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2010 Petr Viktorin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from PyQt4 import QtCore, QtGui
from picard.util import rot13

class OptionUi(object):
    def __init__(self, option, parameters):
        self.option = option
        self.stretch = parameters.pop('stretch', 0)
        self.default_button = parameters.pop('default_button', False)
        self.description = parameters.pop('description', None)
        self.caption = parameters.pop('caption', '')
        # Translate
        if self.caption:
            self.caption = _(self.caption)
        else:
            self.caption = self.option.name

    def _end_init(self, parameters):
        if self.widget:
            self.widget.setObjectName(self.option.name)
        for param in parameters:
            print "Warning - unused option UI parameter: %s (%s)" % (param, self.option.name)

    def addToLayout(self, layout, stretch=None):
        self.addDescription(layout)
        if stretch is None:
            stretch = self.stretch
        if self.default_button:
            myLayout = QtGui.QHBoxLayout()
            myLayout.addWidget(self.widget)
            default_button = QtGui.QPushButton(_("Default"))
            default_button.connect(default_button, QtCore.SIGNAL('clicked()'), self.resetToDefault)
            myLayout.addWidget(default_button)
            layout.addLayout(myLayout)
            added=myLayout
        else:
            layout.addWidget(self.widget)
            added=self.widget
        layout.setStretchFactor(added, stretch)

    def addDescription(self, layout):
        if self.description:
            label = QtGui.QLabel(_(self.description))
            label.setWordWrap(True)
            layout.addWidget(label)

    def resetToDefault(self):
        self.setValue(self.option.default)

class CaptionedOptionUi(OptionUi):
    layout_class = QtGui.QVBoxLayout

    def __init__(self, option, parameters):
        OptionUi.__init__(self, option, parameters)
        horizontal = parameters.pop('horizontal', None)
        if horizontal:
            self.layout_class = QtGui.QHBoxLayout
        elif horizontal is not None:
            self.layout_class = QtGui.QVBoxLayout

    def addToLayout(self, layout):
        myLayout = self.layout_class()
        if self.caption:
            label = QtGui.QLabel(self.caption + ':')
            myLayout.addWidget(label)
            label.setBuddy(self.widget)
        OptionUi.addToLayout(self, myLayout, 0)
        layout.addLayout(myLayout)
        layout.setStretchFactor(myLayout, self.stretch)

class CheckBox(OptionUi):
    def __init__(self, option, parent, parameters):
        OptionUi.__init__(self, option, parameters)
        self.widget = widget = QtGui.QCheckBox(parent)
        widget.setText(self.caption)
        if parameters.pop('inverse', False):
            self._invert = lambda v: not v
        else:
            self._invert = lambda v: v
        OptionUi._end_init(self, parameters)

    def getValue(self):
        return self._invert(self.widget.isChecked())

    def setValue(self, value):
        return self.widget.setChecked(self._invert(value))

class RadioBoxes(OptionUi):
    def __init__(self, option, parent, parameters):
        OptionUi.__init__(self, option, parameters)
        self.widget = widget = QtGui.QGroupBox(parent)
        widget.setTitle(self.caption)
        self.widgets = widgets = {}
        layout = QtGui.QHBoxLayout(widget)
        callback = parameters.pop('callback', None)
        for val_cap in parameters.pop('values'):
            if not isinstance(val_cap, tuple):
                value = caption = val_cap
            else:
                value, caption = val_cap
            radio = QtGui.QRadioButton(widget)
            radio.setText(_(caption))
            layout.addWidget(radio)
            widgets[value] = radio
            if callback:
                if isinstance(callback, basestring):
                    callback = getattr(parent, callback)
                radio.connect(radio, QtCore.SIGNAL("clicked()"), callback)
        layout.addItem(QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum))
        OptionUi._end_init(self, parameters)

    def getValue(self):
        for value, widget in self.widgets.items():
            if widget.isChecked():
                return value

    def setValue(self, value, trying_default = False):
        try:
            self.widgets[value].setChecked(True)
        except KeyError:
            # Not found => try to select the default; if not possible, do nothing
            if trying_default:
                return
            else:
                self.setValue(self.option.default)

    def setOptionEnabled(self, option, enable):
        self.widgets[option].setEnabled(enable)

class LineEntry(CaptionedOptionUi):
    def __init__(self, option, parent, parameters):
        CaptionedOptionUi.__init__(self, option, parameters)
        self.widget = widget = QtGui.QLineEdit(parent)
        self.encode = self.decode = lambda x: x
        try:
            if option.is_password:
                widget.setEchoMode(QtGui.QLineEdit.Password)
                self.encode = self.decode = rot13
        except AttributeError:
            pass
        CaptionedOptionUi._end_init(self, parameters)

    def getValue(self):
        return self.encode(unicode(self.widget.text()))

    def setValue(self, value):
        self.widget.setText(self.decode(value))

class ComboBox(CaptionedOptionUi):
    def __init__(self, option, parent, parameters):
        CaptionedOptionUi.__init__(self, option, parameters)
        self.widget = widget = QtGui.QComboBox(parent)
        self.editable = parameters.pop('editable', True)
        widget.setEditable(self.editable)
        widget.addItems(parameters.pop('values', []))
        CaptionedOptionUi._end_init(self, parameters)

    def getValue(self):
        if self.editable:
            return unicode(self.widget.currentText()).strip()
        else:
            return self.widget.itemData(self.widget.currentIndex()).toString()

    def setValue(self, value):
        if self.editable:
            self.widget.setEditText(value)
        else:
            variant = QtCore.QVariant(value)
            self.widget.setCurrentIndex(self.widget.findData(variant))

class SpinBox(CaptionedOptionUi):
    layout_class = QtGui.QHBoxLayout

    def __init__(self, option, parent, parameters):
        CaptionedOptionUi.__init__(self, option, parameters)
        self.widget = widget = QtGui.QSpinBox(parent)
        suffix = parameters.pop('suffix', None)
        if suffix:
            widget.setSuffix(suffix)
        CaptionedOptionUi._end_init(self, parameters)

    def getValue(self):
        return self.widget.value()

    def setValue(self, value):
        self.widget.setValue(value)

class NoUi(OptionUi):
    def __init__(self, option, parent, parameters):
        OptionUi.__init__(self, option, parameters)
        self.widget = None
        OptionUi._end_init(self, parameters)

    def getValue(self):
        return self._value

    def setValue(self, value):
        self._value = value

class Section(OptionUi):
    def __init__(self, option, parent, parameters):
        OptionUi.__init__(self, option, parameters)
        self.widget = widget = QtGui.QGroupBox(parent)
        widget.setCheckable(True)
        OptionUi._end_init(self, parameters)

    def getValue(self):
        return self.widget.isChecked()

    def setValue(self, value):
        return self.widget.setChecked(value)

    def addToLayout(self, layout):
        self.addDescription(layout)
